package denim;

/*
 * Created by Work on 27/10/2016.
 */

import beast.core.*;
import beast.core.parameter.RealParameter;
import beast.evolution.tree.Tree;
import beast.util.Randomizer;

import java.util.List;
import java.util.Random;

@Description("A gene tree with ploidy and embedding parameters.")

public class GtreeWithEmbedding extends Distribution {
    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<Tree> gtree =
            new Input<>("tree",
                    "A gene tree",
                    Input.Validate.REQUIRED);

    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<Double> coalFactor =
            new Input<>("Ploidy",
                    "Coalescence factor, often the same as ploidy. Default is 2.",
                    2.0);

    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<RealParameter> migPoints =
            new Input<>("migrationPoints",
                    "The positions along branches at which migrations may occur.",
                    Input.Validate.REQUIRED);

    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<RealParameter> destinationChoices =
            new Input<>("destinationChoices",
                    "Numbers which specify which child branches of each internal node specify the destination of a migration.",
                    Input.Validate.REQUIRED);


    @Override
    public List<String> getArguments() {
        return null;
    }

    @Override
    public List<String> getConditions() {
        return null;
    }

    @Override
    public void sample(State state, Random random) {

    }

    @Override
    public void initAndValidate() {
        assert migPoints != null;
        assert migPoints.get() != null;
        int inc = getTree().getInternalNodeCount();
        migPoints.get().setDimension(inc);
        destinationChoices.get().setDimension(inc);


        Double [] initMPArray = new Double[inc];
        Double [] initDCArray = new Double[inc];
        for (int i = 0; i < inc; i++) {
            initMPArray[i] = Randomizer.nextDouble();
            initDCArray[i] = Randomizer.nextDouble();
        }
        RealParameter initMP = new RealParameter(initMPArray);
        migPoints.get().assignFromWithoutID(initMP);
        RealParameter initDC = new RealParameter(initDCArray);
        destinationChoices.get().assignFromWithoutID(initDC);
    }


    boolean somethingIsDirty() {
        return destinationChoices.isDirty() || migPoints.isDirty() || gtree.get().somethingIsDirty();
    }

    public Tree getTree() {
        return gtree.get();
    }

    public double getMigPoint(int idx) {
        return migPoints.get().getValue(idx);
    }

    public double getDestinationChoice(int idx) {
        return destinationChoices.get().getValue(idx);
    }

    double getCoalFactor() {
        return coalFactor.get();
    }
}

